@@ -928,34 +928,34 @@ clearlooks_glossy_draw_scrollbar_stepper (cairo_t *cr,
 	
 	if (scrollbar->horizontal)
 	{
-		if (stepper->stepper == CL_STEPPER_A)
+		if (stepper->stepper == CL_STEPPER_START)
 			corners = CR_CORNER_TOPLEFT | CR_CORNER_BOTTOMLEFT;
-		else if (stepper->stepper == CL_STEPPER_D)
+		else if (stepper->stepper == CL_STEPPER_END)
 			corners = CR_CORNER_TOPRIGHT | CR_CORNER_BOTTOMRIGHT;
 
-		if (stepper->stepper == CL_STEPPER_B)
+		if (stepper->stepper == CL_STEPPER_START_INNER)
 		{
 			x -= 1;
 			width += 1;
 		}
-		else if (stepper->stepper == CL_STEPPER_C)
+		else if (stepper->stepper == CL_STEPPER_END_INNER)
 		{
 			width += 1;
 		}
 	}
 	else
 	{
-		if (stepper->stepper == CL_STEPPER_A)
+		if (stepper->stepper == CL_STEPPER_START)
 			corners = CR_CORNER_TOPLEFT | CR_CORNER_TOPRIGHT;
-		else if (stepper->stepper == CL_STEPPER_D)
+		else if (stepper->stepper == CL_STEPPER_END)
 			corners = CR_CORNER_BOTTOMLEFT | CR_CORNER_BOTTOMRIGHT;
 
-		if (stepper->stepper == CL_STEPPER_B)
+		if (stepper->stepper == CL_STEPPER_START_INNER)
 		{
 			y -= 1;
 			height += 1;
 		}
-		else if (stepper->stepper == CL_STEPPER_C)
+		else if (stepper->stepper == CL_STEPPER_END_INNER)
 		{
 			height += 1;
 		}
