@@ -1,3 +1,26 @@
+/* Clearlooks - a cairo based GTK+ engine
+ * Copyright (C) 2006 Benjamin Berg <benjamin@sipsolutions.net>
+ * Copyright (C) 2007-2008 Andrea Cimitan <andrea.cimitan@gmail.com>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ * Project contact: <gnome-themes-list@gnome.org>
+ *
+ */
+
+
 #ifndef CLEARLOOKS_TYPES_H
 #define CLEARLOOKS_TYPES_H
 
@@ -6,6 +29,7 @@
 typedef unsigned char boolean;
 typedef unsigned char uint8;
 typedef struct _ClearlooksStyleFunctions ClearlooksStyleFunctions;
+typedef struct _ClearlooksStyleConstants ClearlooksStyleConstants;
 
 typedef enum
 {
@@ -13,17 +37,10 @@ typedef enum
 	CL_STYLE_GLOSSY = 1,
 	CL_STYLE_INVERTED = 2,
 	CL_STYLE_GUMMY = 3,
-	CL_NUM_STYLES = 4
+	CL_STYLE_GNOME3 = 4,
+	CL_NUM_STYLES = 5
 } ClearlooksStyles;
 
-typedef enum
-{
-	CL_STATE_NORMAL,
-	CL_STATE_ACTIVE,
-	CL_STATE_SELECTED,
-	CL_STATE_INSENSITIVE
-} ClearlooksStateType;
-
 typedef enum
 {
 	CL_JUNCTION_NONE      = 0,
@@ -33,18 +50,17 @@ typedef enum
 
 typedef enum
 {
-	CL_STEPPER_UNKNOWN    = 0,
-	CL_STEPPER_A          = 1,
-	CL_STEPPER_B          = 2,
-	CL_STEPPER_C          = 4,
-	CL_STEPPER_D          = 8
+	CL_STEPPER_UNKNOWN,
+	CL_STEPPER_START,
+	CL_STEPPER_START_INNER,
+	CL_STEPPER_END,
+	CL_STEPPER_END_INNER,
 } ClearlooksStepper;
 
 typedef enum
 {
-	CL_ORDER_FIRST,
-	CL_ORDER_MIDDLE,
-	CL_ORDER_LAST
+	CL_ORDER_FIRST    = 1 << 0,
+	CL_ORDER_LAST     = 1 << 1,
 } ClearlooksOrder;
 
 typedef enum
@@ -104,6 +120,7 @@ typedef enum
 	CL_FOCUS_TAB,
 	CL_FOCUS_COLOR_WHEEL_DARK,
 	CL_FOCUS_COLOR_WHEEL_LIGHT,
+	CL_FOCUS_ICONVIEW,
 	CL_FOCUS_UNKNOWN
 } ClearlooksFocusType;
 
@@ -165,7 +182,7 @@ typedef struct
 
 	gfloat  radius;
 	
-	ClearlooksStateType state_type;
+	GtkStateType state_type;
 	
 	uint8 corners;
 	uint8 xthickness;
@@ -174,6 +191,7 @@ typedef struct
 	CairoColor parentbg;
 
 	ClearlooksStyleFunctions *style_functions;
+	ClearlooksStyleConstants *style_constants;
 } WidgetParameters;
 
 typedef struct
@@ -275,6 +293,18 @@ typedef struct
 	boolean              in_menu;
 } CheckboxParameters;
 
+typedef struct
+{
+	/* The maximum size of the fill. Calcualted from the entries allocation,
+	 * and other information. Relative to the drawn position.
+	 */
+	GdkRectangle max_size;
+	gboolean max_size_known;
+	/* The border around the bar. This can be used for radius calculations.
+	 */
+	GtkBorder border;
+} EntryProgressParameters;
+
 typedef struct
 {
 	ClearlooksArrowType type;
@@ -287,8 +317,21 @@ typedef struct
 	boolean  topmost;
 } ToolbarParameters;
 
+struct _ClearlooksStyleConstants
+{
+	gdouble topleft_highlight_shade;
+	gdouble topleft_highlight_alpha;
+};
+
 struct _ClearlooksStyleFunctions
 {
+	void (*draw_top_left_highlight) (cairo_t *cr,
+	                               const CairoColor *color,
+	                               const WidgetParameters *params,
+                                       int x, int y, int width, int height,
+                                       gdouble radius,
+                                       CairoCorners corners);
+
 	void (*draw_button)           (cairo_t *cr,
 	                               const ClearlooksColors *colors,
 	                               const WidgetParameters *widget,
@@ -320,6 +363,13 @@ struct _ClearlooksStyleFunctions
 	void (*draw_entry)            (cairo_t *cr,
 	                               const ClearlooksColors *colors,
 	                               const WidgetParameters *widget,
+	                               const FocusParameters  *focus,
+	                               int x, int y, int width, int height);
+
+	void (*draw_entry_progress)   (cairo_t *cr,
+	                               const ClearlooksColors *colors,
+	                               const WidgetParameters *widget,
+	                               const EntryProgressParameters *progress,
 	                               int x, int y, int width, int height);
 
 	void (*draw_spinbutton)       (cairo_t *cr,
@@ -470,6 +520,11 @@ struct _ClearlooksStyleFunctions
 	                               const CheckboxParameters	*checkbox,
 	                               int x, int y, int width, int height);
 
+	void (*draw_icon_view_item)   (cairo_t *cr,
+	                               const ClearlooksColors *colors,
+	                               const WidgetParameters *widget,
+	                               int x, int y, int width, int height);
+
 	/* Style internal functions */
 	/* XXX: Only used by slider_button, inline it? */
 	void (*draw_shadow)           (cairo_t *cr,
@@ -488,9 +543,9 @@ struct _ClearlooksStyleFunctions
 };
 
 
-#define CLEARLOOKS_RECTANGLE_SET(rect, _x, _y, _w, _h) rect.x      = _x; \
-                                                       rect.y      = _y; \
-                                                       rect.width  = _w; \
-                                                       rect.height = _h;
+#define CLEARLOOKS_RECTANGLE_SET(rect, _x, _y, _w, _h) (rect).x      = (_x); \
+                                                       (rect).y      = (_y); \
+                                                       (rect).width  = (_w); \
+                                                       (rect).height = (_h);
 
 #endif /* CLEARLOOKS_TYPES_H */
