@@ -20,44 +20,39 @@ static GtkStyleClass *parent_style_class;
 
 static void
 draw_hline (GtkStyle *style,
-	    GdkWindow *window,
+	    cairo_t *cr,
 	    GtkStateType state_type,
-	    GdkRectangle *area,
 	    GtkWidget *widget,
 	    const gchar *detail,
 	    gint x1, gint x2, gint y);
 
 static void
 draw_vline (GtkStyle *style,
-	    GdkWindow *window,
+	    cairo_t *cr,
 	    GtkStateType state_type,
-	    GdkRectangle *area,
 	    GtkWidget *widget, const gchar *detail, gint y1, gint y2, gint x);
 
 static void
 draw_shadow (GtkStyle *style,
-	     GdkWindow *window,
+	     cairo_t *cr,
 	     GtkStateType state_type,
 	     GtkShadowType shadow_type,
-	     GdkRectangle *area,
 	     GtkWidget *widget,
 	     const gchar *detail, gint x, gint y, gint width, gint height);
 
 static void
 draw_box (GtkStyle *style,
-	  GdkWindow *window,
+	  cairo_t *cr,
 	  GtkStateType state_type,
 	  GtkShadowType shadow_type,
-	  GdkRectangle *area,
 	  GtkWidget *widget,
 	  const gchar *detail, gint x, gint y, gint width, gint height);
 
 static void
 draw_arrow (GtkStyle *style,
-	    GdkWindow *window,
+	    cairo_t *cr,
 	    GtkStateType state_type,
 	    GtkShadowType shadow_type,
-	    GdkRectangle *area,
 	    GtkWidget *widget,
 	    const gchar *detail,
 	    GtkArrowType arrow_type,
@@ -65,36 +60,32 @@ draw_arrow (GtkStyle *style,
 
 static void
 draw_check (GtkStyle *style,
-	    GdkWindow *window,
+	    cairo_t *cr,
 	    GtkStateType state_type,
 	    GtkShadowType shadow_type,
-	    GdkRectangle *area,
 	    GtkWidget *widget,
 	    const gchar *detail, gint x, gint y, gint width, gint height);
 static void
 draw_option (GtkStyle *style,
-	     GdkWindow *window,
+	     cairo_t *cr,
 	     GtkStateType state_type,
 	     GtkShadowType shadow_type,
-	     GdkRectangle *area,
 	     GtkWidget *widget,
 	     const gchar *detail, gint x, gint y, gint width, gint height);
 
 static void
 draw_tab (GtkStyle *style,
-	  GdkWindow *window,
+	  cairo_t *cr,
 	  GtkStateType state_type,
 	  GtkShadowType shadow_type,
-	  GdkRectangle *area,
 	  GtkWidget *widget,
 	  const gchar *detail, gint x, gint y, gint width, gint height);
 
 static void
 draw_shadow_gap (GtkStyle *style,
-		 GdkWindow *window,
+	         cairo_t *cr,
 		 GtkStateType state_type,
 		 GtkShadowType shadow_type,
-		 GdkRectangle *area,
 		 GtkWidget *widget,
 		 const gchar *detail,
 		 gint x,
@@ -105,10 +96,9 @@ draw_shadow_gap (GtkStyle *style,
 
 static void
 draw_box_gap (GtkStyle *style,
-	      GdkWindow *window,
+	      cairo_t *cr,
 	      GtkStateType state_type,
 	      GtkShadowType shadow_type,
-	      GdkRectangle *area,
 	      GtkWidget *widget,
 	      const gchar *detail,
 	      gint x,
@@ -119,10 +109,9 @@ draw_box_gap (GtkStyle *style,
 
 static void
 draw_extension (GtkStyle *style,
-		GdkWindow *window,
+	        cairo_t *cr,
 		GtkStateType state_type,
 		GtkShadowType shadow_type,
-		GdkRectangle *area,
 		GtkWidget *widget,
 		const gchar *detail,
 		gint x,
@@ -130,9 +119,8 @@ draw_extension (GtkStyle *style,
 
 static void
 draw_focus (GtkStyle *style,
-	    GdkWindow *window,
+	    cairo_t *cr,
 	    GtkStateType state_type,
-	    GdkRectangle *area,
 	    GtkWidget *widget,
 	    const gchar *detail,
 	    gint x,
@@ -142,10 +130,9 @@ draw_focus (GtkStyle *style,
 
 static void
 draw_slider (GtkStyle *style,
-	     GdkWindow *window,
+	     cairo_t *cr,
 	     GtkStateType state_type,
 	     GtkShadowType shadow_type,
-	     GdkRectangle *area,
 	     GtkWidget *widget,
 	     const gchar *detail,
 	     gint x,
@@ -153,10 +140,9 @@ draw_slider (GtkStyle *style,
 
 static void
 draw_handle (GtkStyle *style,
-	     GdkWindow *window,
+	     cairo_t *cr,
 	     GtkStateType state_type,
 	     GtkShadowType shadow_type,
-	     GdkRectangle *area,
 	     GtkWidget *widget,
 	     const gchar *detail,
 	     gint x,
@@ -625,7 +611,7 @@ paint_scale_trough (cairo_t *cr, GtkStyle *style, GtkStateType state_type, GtkOr
 }
 
 static void
-paint_progress_bar (cairo_t *cr, GtkStyle *style, GtkStateType state_type, GtkProgressBarOrientation orientation,
+paint_progress_bar (cairo_t *cr, GtkStyle *style, GtkStateType state_type, GtkOrientation orientation, gboolean inverted,
 		    gdouble x, gdouble y, gdouble width, gdouble height, gdouble fraction, CairoColor *base_color)
 {
 	cairo_pattern_t *crp;
@@ -636,7 +622,7 @@ paint_progress_bar (cairo_t *cr, GtkStyle *style, GtkStateType state_type, GtkPr
 	ge_gdk_color_to_cairo (&style->base[GTK_STATE_SELECTED], &c1);
 	CRUX_DARK(*base_color, c2);
 	CRUX_LIGHT(*base_color, c1);
-	if (orientation == GTK_PROGRESS_LEFT_TO_RIGHT || orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
+	if (orientation == GTK_ORIENTATION_HORIZONTAL)
 		crp = cairo_pattern_create_linear (x, y, x, y + height);
 	else
 		crp = cairo_pattern_create_linear (x, y, x + width, y);
@@ -653,13 +639,22 @@ paint_progress_bar (cairo_t *cr, GtkStyle *style, GtkStateType state_type, GtkPr
 
 	if (fraction < 1.0)
 	{
-		switch (orientation)
-		{
-			case GTK_PROGRESS_LEFT_TO_RIGHT: width--; break;
-			case GTK_PROGRESS_RIGHT_TO_LEFT: x++; width--; break;
-			case GTK_PROGRESS_TOP_TO_BOTTOM: height--; break;
-			case GTK_PROGRESS_BOTTOM_TO_TOP: y++; height--; break;
+		if (orientation == GTK_ORIENTATION_HORIZONTAL && !inverted)
+                {
+                        width--;
+                }
+                else if (orientation == GTK_ORIENTATION_HORIZONTAL && inverted)
+                {
+		        x++; width--;
+                }
+                else if (orientation == GTK_ORIENTATION_VERTICAL && !inverted)
+                {
+		        height--;
 		}
+                else if (orientation == GTK_ORIENTATION_VERTICAL && inverted)
+		{
+		        y++; height--;
+                }
 	}
 
 
@@ -680,24 +675,25 @@ paint_progress_bar (cairo_t *cr, GtkStyle *style, GtkStateType state_type, GtkPr
 	if (fraction < 1.0)
 	{
 		cairo_set_source_rgb (cr, c2.r, c2.g, c2.b);
-		switch (orientation)
-		{
-			case GTK_PROGRESS_LEFT_TO_RIGHT:
-				cairo_move_to (cr, x + width + 1.0, y);
-				cairo_line_to (cr, x + width + 1.0, y + height);
-				break;
-			case GTK_PROGRESS_RIGHT_TO_LEFT:
+                if (orientation == GTK_ORIENTATION_HORIZONTAL && !inverted)
+                {
+			cairo_move_to (cr, x + width + 1.0, y);
+			cairo_line_to (cr, x + width + 1.0, y + height);
+                }
+                else if (orientation == GTK_ORIENTATION_HORIZONTAL && inverted)
+                {
 				cairo_move_to (cr, x - 1.0, y);
 				cairo_line_to (cr, x - 1.0, y + height);
-				break;
-			case GTK_PROGRESS_TOP_TO_BOTTOM:
+                }
+                else if (orientation == GTK_ORIENTATION_VERTICAL && !inverted)
+                {
 				cairo_move_to (cr, x, y + height + 1.0);
 				cairo_line_to (cr, x + width, y + height + 1.0);
-				break;
-			case GTK_PROGRESS_BOTTOM_TO_TOP:
+                }
+                else if (orientation == GTK_ORIENTATION_VERTICAL && inverted)
+                {
 				cairo_move_to (cr, x, y - 1.0);
 				cairo_line_to (cr, x + width, y - 1.0);
-				break;
 		}
 		cairo_stroke (cr);
 	}
@@ -711,7 +707,7 @@ paint_progress_bar (cairo_t *cr, GtkStyle *style, GtkStateType state_type, GtkPr
 	 *       is already drawn by draw_box (), so the corner looks too dark
 	 */
 
-	if (orientation == GTK_PROGRESS_LEFT_TO_RIGHT)
+	if (orientation == GTK_ORIENTATION_HORIZONTAL && !inverted)
 	{
 		cairo_set_source_rgba (cr, 0, 0, 0, .22);
 		cairo_move_to (cr, x + width + 2.0, y);
@@ -734,22 +730,19 @@ paint_progress_bar (cairo_t *cr, GtkStyle *style, GtkStateType state_type, GtkPr
 
 static void
 draw_hline (GtkStyle *style,
-	    GdkWindow *window,
+            cairo_t *cr,
 	    GtkStateType state_type,
-	    GdkRectangle *area,
 	    GtkWidget *widget,
 	    const gchar *detail,
 	    gint x1, gint x2, gint y)
 {
 	CairoColor base, light, dark;
-	cairo_t *cr;
 
 	CHECK_ARGS
 
 	debug ("draw_hline: detail=%s state=%d x1=%d x2=%d y=%d\n",
 		detail, state_type, x1, x2, y);
 
-	cr = ge_gdk_drawable_to_cairo (window, area);
 	ge_gdk_color_to_cairo (&style->bg[state_type], &base);
 	ge_shade_color (&base, 0.88, &dark);
 	ge_shade_color (&base, 1.12, &light);
@@ -762,25 +755,21 @@ draw_hline (GtkStyle *style,
 	cairo_move_to (cr, x1 + 0.5, y + 0.5);
 	cairo_line_to (cr, x2 + 0.5, y + 0.5);
 	cairo_stroke (cr);
-	cairo_destroy (cr);
 }
 
 static void
 draw_vline (GtkStyle *style,
-	    GdkWindow *window,
+            cairo_t *cr,
 	    GtkStateType state_type,
-	    GdkRectangle *area,
 	    GtkWidget *widget, const gchar *detail, gint y1, gint y2, gint x)
 {
 	CairoColor base, light, dark;
-	cairo_t *cr;
 
 	CHECK_ARGS
 
 	debug ("draw_vline: detail=%s state=%d x=%d y1=%d y2=%d\n",
 		detail, state_type, x, y1, y2);
 
-	cr = ge_gdk_drawable_to_cairo (window, area);
 	ge_gdk_color_to_cairo (&style->bg[state_type], &base);
 	ge_shade_color (&base, 0.88, &dark);
 	ge_shade_color (&base, 1.12, &light);
@@ -793,29 +782,19 @@ draw_vline (GtkStyle *style,
 	cairo_move_to (cr, x + 0.5, y1 + 0.5);
 	cairo_line_to (cr, x + 0.5, y2 + 0.5);
 	cairo_stroke (cr);
-	cairo_destroy (cr);
 }
 
 static void
 draw_shadow (GtkStyle *style,
-	     GdkWindow *window,
+	     cairo_t *cr,
 	     GtkStateType state_type,
 	     GtkShadowType shadow_type,
-	     GdkRectangle *area,
 	     GtkWidget *widget,
 	     const gchar *detail, gint x, gint y, gint width, gint height)
 {
-	cairo_t *cr;
-	GdkRectangle area2;
 	gboolean focused = FALSE;
 
 	CHECK_ARGS
-	SANITIZE_SIZE
-
-	area2.x = x;
-	area2.y = y;
-	area2.width = width;
-	area2.height = height;
 
 	if (widget != 0 && !gtk_widget_get_sensitive (widget))
 		state_type = GTK_STATE_INSENSITIVE;
@@ -837,8 +816,6 @@ draw_shadow (GtkStyle *style,
 			width += 3;
 		}
 
-		if (area == NULL)
-			area = &area2;
 		g_object_set_data ((GObject*) gtk_widget_get_parent (widget), "entry", widget);
 		button = g_object_get_data ((GObject*) gtk_widget_get_parent (widget), "button");
 		if (GE_IS_WIDGET (button))
@@ -854,41 +831,30 @@ draw_shadow (GtkStyle *style,
 			x -= 3;
 			width += 3;
 		}
-
-		if (area == NULL)
-			area = &area2;
 	}
 	
-	cr = ge_gdk_drawable_to_cairo (window, area);
-
 	if (DETAIL ("entry"))
 		paint_entry_shadow (cr, style, state_type, focused, x, y, width, height);
 	else
 		paint_shadow (cr, style, state_type, shadow_type, x, y, width, height);
-	cairo_destroy (cr);
 }
 
 static void
 draw_box (GtkStyle *style,
-	  GdkWindow *window,
+	  cairo_t *cr,
 	  GtkStateType state_type,
 	  GtkShadowType shadow_type,
-	  GdkRectangle *area,
 	  GtkWidget *widget,
 	  const gchar *detail, gint x, gint y, gint width, gint height)
 {
 
-	cairo_t *cr;
 	gdouble cx, cy, cw, ch;
 
 	CHECK_ARGS
-	SANITIZE_SIZE
 
     debug ("draw_box: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
 	    detail, state_type, shadow_type, x, y, width, height);
 
-	cr = ge_gdk_drawable_to_cairo (window, area);
-
 	if (DETAIL ("spinbutton"))
 	{
 		gboolean focused = FALSE;
@@ -902,7 +868,6 @@ draw_box (GtkStyle *style,
 
 		paint_entry_shadow (cr, style, state_type, focused, x, y, width, height);
 
-		cairo_destroy (cr);
 		return;
 	}
 
@@ -1022,23 +987,25 @@ draw_box (GtkStyle *style,
 	}
 	else if (DETAIL ("bar") || DETAIL ("entry-progress"))
 	{
-		GtkProgressBarOrientation orientation;
+		GtkOrientation orientation;
+                gboolean inverted = FALSE;
 		CairoColor base_color;
 		gdouble fraction = 0;
 		if (widget && GE_IS_PROGRESS_BAR (widget))
 		{
-			orientation = gtk_progress_bar_get_orientation (GTK_PROGRESS_BAR (widget));
+			orientation = gtk_orientable_get_orientation (GTK_ORIENTABLE (widget));
+                        inverted = gtk_progress_bar_get_inverted (GTK_PROGRESS_BAR (widget));
 			fraction = gtk_progress_bar_get_fraction (GTK_PROGRESS_BAR (widget));
 		}
 		else
-			orientation = GTK_PROGRESS_LEFT_TO_RIGHT;
+			orientation = GTK_ORIENTATION_HORIZONTAL;
 
 		if (DETAIL ("bar"))
 			ge_gdk_color_to_cairo (&style->base[GTK_STATE_SELECTED], &base_color);
 		else
 			ge_gdk_color_to_cairo (&style->bg[state_type], &base_color);
 
-		paint_progress_bar (cr, style, state_type, orientation, x, y, width, height, fraction, &base_color);
+		paint_progress_bar (cr, style, state_type, orientation, inverted, x, y, width, height, fraction, &base_color);
 	}
 	else if (DETAIL ("trough") && GE_IS_SCALE (widget))
 	{
@@ -1171,24 +1138,18 @@ draw_box (GtkStyle *style,
 
 		paint_shadow (cr, style, state_type, shadow_type, x, y, width, height);
 	}
-
-	cairo_destroy (cr);
-
 }
 
 static void
 draw_arrow (GtkStyle *style,
-	    GdkWindow *window,
+	    cairo_t *cr,
 	    GtkStateType state_type,
 	    GtkShadowType shadow_type,
-	    GdkRectangle *area,
 	    GtkWidget *widget,
 	    const gchar *detail,
 	    GtkArrowType arrow_type,
 	    gint fill, gint x, gint y, gint width, gint height)
 {
-	cairo_t *cr;
-
 	CHECK_ARGS;
 	debug ("draw_arrow: detail=%s state=%d shadow=%d arrow_type=%d x=%d y=%d w=%d h=%d\n",
 	    detail, state_type, shadow_type, arrow_type, x, y, width, height);
@@ -1229,8 +1190,6 @@ draw_arrow (GtkStyle *style,
 	}
 
 
-
-	cr = ge_gdk_drawable_to_cairo (window, area);
 	switch (arrow_type)
 	{
 		case GTK_ARROW_UP:
@@ -1262,33 +1221,25 @@ draw_arrow (GtkStyle *style,
 
 	gdk_cairo_set_source_color (cr, &style->fg[state_type]);
 	cairo_fill (cr);
-	
-	cairo_destroy (cr);
-
 }
 
 static void
 draw_check (GtkStyle *style,
-	    GdkWindow *window,
+	    cairo_t *cr,
 	    GtkStateType state_type,
 	    GtkShadowType shadow_type,
-	    GdkRectangle *area,
 	    GtkWidget *widget,
 	    const gchar *detail, gint x, gint y, gint width, gint height)
 {
-	cairo_t * cr;
 	cairo_pattern_t *crp;
 	CairoColor c1, bg, outline;
 	gdouble cx, cy, cw, ch; /* co-ordinates for cairo */
 
 	CHECK_ARGS
-	SANITIZE_SIZE
 	
 	debug ("draw_check: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
 	    detail, state_type, shadow_type, x, y, width, height);
 
-	cr = ge_gdk_drawable_to_cairo (window, NULL);
-
 	/* set up colours */
 	ge_gdk_color_to_cairo (&style->bg[GTK_STATE_NORMAL], &bg);
 	/* outline colour */
@@ -1386,32 +1337,25 @@ draw_check (GtkStyle *style,
 		cairo_rectangle (cr, x + width / 4, y + (height /3), width - (width / 2), height / 4);
 		cairo_fill (cr);
 	}
-
-	cairo_destroy (cr);
 }
 
 static void
 draw_option (GtkStyle *style,
-	     GdkWindow *window,
+	     cairo_t *cr,
 	     GtkStateType state_type,
 	     GtkShadowType shadow_type,
-	     GdkRectangle *area,
 	     GtkWidget *widget,
 	     const gchar *detail, gint x, gint y, gint width, gint height)
 {
-	cairo_t *cr;
 	cairo_pattern_t *crp;
 	gdouble cx, cy, radius; /* cairo co-ordinates */
 	CairoColor c1, c2, bg, outline;
 
 	CHECK_ARGS
-	SANITIZE_SIZE
 	
 	debug ("draw_option: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
 	    detail, state_type, shadow_type, x, y, width, height);
 
-	cr = ge_gdk_drawable_to_cairo (window, area);
-
 	/* set up colours */
 	ge_gdk_color_to_cairo (&style->bg[GTK_STATE_NORMAL], &bg);
 	/* outline colour */
@@ -1497,23 +1441,17 @@ draw_option (GtkStyle *style,
 		cairo_rectangle (cr, x + width / 4, y + (height /3), width - (width / 2), height / 4);
 		cairo_fill (cr);
 	}
-
-
-
-	cairo_destroy (cr);
 }
 
 static void
 draw_tab (GtkStyle *style,
-	  GdkWindow *window,
+	  cairo_t *cr,
 	  GtkStateType state_type,
 	  GtkShadowType shadow_type,
-	  GdkRectangle *area,
 	  GtkWidget *widget,
 	  const gchar *detail, gint x, gint y, gint width, gint height)
 {
     CHECK_ARGS
-    SANITIZE_SIZE
 
 	/* Draws an option menu tab (the up and down pointing arrows)
 	 * TODO: Make this look neater
@@ -1521,19 +1459,18 @@ draw_tab (GtkStyle *style,
 	debug ("draw_tab: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
 	    detail, state_type, shadow_type, x, y, width, height);
 
-	draw_arrow (style, window, state_type, shadow_type, area, widget, detail,
+	draw_arrow (style, cr, state_type, shadow_type, widget, detail,
 		    GTK_ARROW_UP, FALSE, x + (width / 2) - 2, y - (height / 2), 7, 7);
 
-	draw_arrow (style, window, state_type, shadow_type, area, widget, detail,
+	draw_arrow (style, cr, state_type, shadow_type, widget, detail,
 		    GTK_ARROW_DOWN, FALSE, x + (width / 2) - 2, y + (height / 2), 7, 7);
 }
 
 static void
 draw_shadow_gap (GtkStyle *style,
-		 GdkWindow *window,
+		 cairo_t *cr,
 		 GtkStateType state_type,
 		 GtkShadowType shadow_type,
-		 GdkRectangle *area,
 		 GtkWidget *widget,
 		 const gchar *detail,
 		 gint x,
@@ -1545,13 +1482,12 @@ draw_shadow_gap (GtkStyle *style,
     GdkRectangle rect;
 
     CHECK_ARGS
-    SANITIZE_SIZE
 
     debug ("draw_shadow_gap: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
 	    detail, state_type, shadow_type, x, y, width, height);
 
-    gtk_paint_shadow (style, window, state_type, shadow_type, area, widget,
-		      detail, x, y, width, height);
+    gtk_paint_shadow (style, cr, state_type, shadow_type, widget,
+		            detail, x, y, width, height);
 
     switch (gap_side)
     {
@@ -1582,16 +1518,15 @@ draw_shadow_gap (GtkStyle *style,
 	break;
     }
 
-    gtk_style_apply_default_background (style, window, TRUE, state_type, area,
+    gtk_style_apply_default_background (style, cr, gtk_widget_get_window (widget), state_type,
 				    rect.x, rect.y, rect.width, rect.height);
 }
 
 static void
 draw_box_gap (GtkStyle *style,
-	      GdkWindow *window,
+	      cairo_t *cr,
 	      GtkStateType state_type,
 	      GtkShadowType shadow_type,
-	      GdkRectangle *area,
 	      GtkWidget *widget,
 	      const gchar *detail,
 	      gint x,
@@ -1603,13 +1538,12 @@ draw_box_gap (GtkStyle *style,
     GdkRectangle rect;
 
     CHECK_ARGS
-    SANITIZE_SIZE
 
 	debug ("draw_box_gap: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
 		detail, state_type, shadow_type, x, y, width, height);
 
-	gtk_paint_box (style, window, state_type, shadow_type, area, widget,
-			detail, x, y, width, height);
+	gtk_paint_box (style, cr, state_type, shadow_type, widget,
+			     detail, x, y, width, height);
 
 
 	switch (gap_side)
@@ -1641,22 +1575,20 @@ draw_box_gap (GtkStyle *style,
 			break;
 	}
 
-	gtk_style_apply_default_background (style, window, TRUE, state_type, area,
+	gtk_style_apply_default_background (style, cr, gtk_widget_get_window (widget), state_type,
 				    rect.x, rect.y, rect.width, rect.height);
 }
 
 static void
 draw_extension (GtkStyle *style,
-		GdkWindow *window,
+                cairo_t *cr,
 		GtkStateType state_type,
 		GtkShadowType shadow_type,
-		GdkRectangle *area,
 		GtkWidget *widget,
 		const gchar *detail,
 		gint x,
 		gint y, gint width, gint height, GtkPositionType gap_side)
 {
-	cairo_t *cr;
 	cairo_pattern_t *crp;
 	CairoColor c1, c2, bg, outline;
 	CairoCorners corners;
@@ -1665,8 +1597,6 @@ draw_extension (GtkStyle *style,
 	debug ("draw_extension: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
 		detail, state_type, shadow_type, x, y, width, height);
 
-	cr = ge_gdk_drawable_to_cairo (window, area);
-
 	/* set up colours */
 	ge_gdk_color_to_cairo (&style->bg[state_type], &bg);
 	ge_shade_color (&bg, 1.1, &c1);
@@ -1802,17 +1732,12 @@ draw_extension (GtkStyle *style,
 		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.5); 
 		cairo_stroke (cr);
 	}
-
-
-
-	cairo_destroy (cr);
 }
 
 static void
 draw_focus (GtkStyle *style,
-	    GdkWindow *window,
+	    cairo_t *cr,
 	    GtkStateType state_type,
-	    GdkRectangle *area,
 	    GtkWidget *widget,
 	    const gchar *detail, gint x, gint y, gint width, gint height)
 {
@@ -1821,12 +1746,9 @@ draw_focus (GtkStyle *style,
 
 	if (DETAIL ("entry"))
 	{
-		cairo_t *cr;
-		cr = ge_gdk_drawable_to_cairo (window, area);
 		cairo_rectangle (cr, x + 0.5, y + 0.5, width - 1.0, height - 1.0);
 		gdk_cairo_set_source_color (cr, &style->base[GTK_STATE_SELECTED]);
 		cairo_stroke (cr);
-		cairo_destroy (cr);
 		return;
 	}
 
@@ -1835,33 +1757,28 @@ draw_focus (GtkStyle *style,
 		return;
 
 	/* Make sure no widget is without a focus indicator! */
-	parent_style_class->draw_focus(style, window, state_type, area, widget,
+	parent_style_class->draw_focus(style, cr, state_type, widget,
 	                               detail, x, y, width, height);
 }
 
 static void
 draw_slider (GtkStyle *style,
-	     GdkWindow *window,
+	     cairo_t *cr,
 	     GtkStateType state_type,
 	     GtkShadowType shadow_type,
-	     GdkRectangle *area,
 	     GtkWidget *widget,
 	     const gchar *detail,
 	     gint x,
 	     gint y, gint width, gint height, GtkOrientation orientation)
 {
-	cairo_t *cr;
 	cairo_pattern_t *crp;
 	CairoColor c1, c2;
 	gboolean is_scale = (DETAIL ("vscale") || DETAIL ("hscale"));
 
 	CHECK_ARGS
-	SANITIZE_SIZE
 
 	debug ("draw_slider: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n", detail, state_type, shadow_type, x, y, width, height);
 
-	cr = ge_gdk_drawable_to_cairo (window, area);
-
 
 	ge_gdk_color_to_cairo (&style->bg[GTK_STATE_SELECTED], &c2);
 
@@ -1917,17 +1834,13 @@ draw_slider (GtkStyle *style,
 	cairo_stroke (cr);
 
 	cairo_pattern_destroy (crp);
-
-
-	cairo_destroy (cr);
 }
 
 static void
 draw_handle (GtkStyle *style,
-	     GdkWindow *window,
+	     cairo_t *cr,
 	     GtkStateType state_type,
 	     GtkShadowType shadow_type,
-	     GdkRectangle *area,
 	     GtkWidget *widget,
 	     const gchar *detail,
 	     gint x,
@@ -1935,11 +1848,9 @@ draw_handle (GtkStyle *style,
 {
     gint i, yy, xx;
     gint xthick, ythick;
-    GdkGC *light_gc, *dark_gc;
     GdkRectangle dest;
 
     CHECK_ARGS
-    SANITIZE_SIZE
 
     debug ("draw_handle: detail=%s state=%d shadow=%d x=%d y=%d w=%d h=%d\n",
 	   detail, state_type, shadow_type, x, y, width, height);
@@ -1948,9 +1859,6 @@ draw_handle (GtkStyle *style,
 	state_type = GTK_STATE_ACTIVE;
 
 
-    light_gc = style->light_gc[state_type];
-    dark_gc = style->dark_gc[state_type];
-
     xthick = style->xthickness + 1;
     ythick = style->ythickness + 1;
 
@@ -1959,8 +1867,9 @@ draw_handle (GtkStyle *style,
     dest.width = width - (xthick * 2);
     dest.height = height - (ythick * 2);
 
-    gdk_gc_set_clip_rectangle (light_gc, &dest);
-    gdk_gc_set_clip_rectangle (dark_gc, &dest);
+    cairo_set_line_width (cr, 1.0);
+    /* trick so we don't have to add 0.5 to all the line calls below */
+    cairo_translate (cr, 0.5, 0.5);
 
     /* ORIENTATION parameters is unreliable */
     if (height > width)
@@ -1968,10 +1877,15 @@ draw_handle (GtkStyle *style,
 	yy = y + height / 2 - 4;
 	for (i = 0; i < 8; i += 2)
 	{
-	    gdk_draw_line (window, dark_gc, xthick, yy + i,
-			   x + width - xthick, yy + i);
-	    gdk_draw_line (window, light_gc, xthick, yy + i + 1,
-			   x + width - xthick, yy + i + 1);
+            gdk_cairo_set_source_color (cr, &style->dark[state_type]);
+            cairo_move_to (cr, xthick, yy + i);
+            cairo_line_to (cr, x + width - xthick, yy + i);
+            cairo_stroke (cr);
+
+            gdk_cairo_set_source_color (cr, &style->light[state_type]);
+            cairo_move_to (cr, xthick, yy + i + 1);
+            cairo_line_to (cr, x + width - xthick, yy + i + 1);
+            cairo_stroke (cr);
 	}
     }
     else
@@ -1979,45 +1893,39 @@ draw_handle (GtkStyle *style,
 	xx = x + width / 2 - 4;
 	for (i = 0; i < 8; i += 2)
 	{
-	    gdk_draw_line (window, dark_gc, xx + i, ythick,
-			   xx + i, y + height - ythick);
-	    gdk_draw_line (window, light_gc, xx + i + 1, ythick,
-			   xx + i + 1, y + height - ythick);
+            gdk_cairo_set_source_color (cr, &style->dark[state_type]);
+            cairo_move_to (cr, xx + i, ythick);
+            cairo_line_to (cr, xx + i, y + height - ythick);
+            cairo_stroke (cr);
+
+            gdk_cairo_set_source_color (cr, &style->light[state_type]);
+            cairo_move_to (cr, xx + i + 1, ythick);
+            cairo_line_to (cr, xx + i + 1, y + height - ythick);
+            cairo_stroke (cr);
 	}
     }
-
-    gdk_gc_set_clip_rectangle (light_gc, NULL);
-    gdk_gc_set_clip_rectangle (dark_gc, NULL);
 }
 
 static void
 draw_layout (GtkStyle        *style,
-	     GdkWindow       *window,
+	     cairo_t         *cr,
 	     GtkStateType     state_type,
 	     gboolean         use_text,
-	     GdkRectangle    *area,
 	     GtkWidget       *widget,
 	     const char      *detail,
 	     int              x,
 	     int              y,
 	     PangoLayout      *layout)
 {
-	GdkGC *gc;
+	CHECK_ARGS
+
+	if (use_text)
+		gdk_cairo_set_source_color (cr, &style->text[state_type]);
+	else
+		gdk_cairo_set_source_color (cr, &style->fg[state_type]);
 	
-	g_return_if_fail (GTK_IS_STYLE (style));
-	g_return_if_fail (window != NULL);
-	
-	gc = use_text ? style->text_gc[state_type] : style->fg_gc[state_type];
-	
-	if (area) {
-		gdk_gc_set_clip_rectangle (gc, area);
-	}
-	
-	gdk_draw_layout (window, gc, x, y, layout);
-	
-	if (area) {
-		gdk_gc_set_clip_rectangle (gc, NULL);
-	}
+	ge_cairo_transform_for_layout (cr, layout, x, y);
+	pango_cairo_show_layout (cr, layout);
 }
 
 
