@@ -22,10 +22,9 @@
 
 GE_INTERNAL void
 glide_draw_shadow_gap (GtkStyle * style, 
-                 GdkWindow * window, 
+                 cairo_t * cr, 
 		 GtkStateType state_type, 
 		 GtkShadowType shadow_type, 
-		 GdkRectangle * area, 
 		 GtkWidget * widget, 
 		 const gchar * detail, 
 		 gint x, 
@@ -38,10 +37,9 @@ glide_draw_shadow_gap (GtkStyle * style,
 
 GE_INTERNAL void
 glide_draw_box_gap (GtkStyle * style, 
-                 GdkWindow * window, 
+                 cairo_t * cr, 
 		 GtkStateType state_type, 
 		 GtkShadowType shadow_type, 
-		 GdkRectangle * area, 
 		 GtkWidget * widget, 
 		 const gchar * detail, 
 		 gint x, 
@@ -54,9 +52,8 @@ glide_draw_box_gap (GtkStyle * style,
 
 GE_INTERNAL void
 glide_draw_hline (GtkStyle * style,
-	    GdkWindow * window,
+	    cairo_t * cr,
 	    GtkStateType state_type,
-	    GdkRectangle * area,
 	    GtkWidget * widget,
 	    const gchar * detail, 
             gint x1,
@@ -65,9 +62,8 @@ glide_draw_hline (GtkStyle * style,
  
 GE_INTERNAL void
 glide_draw_vline (GtkStyle * style,
-	    GdkWindow * window,
+	    cairo_t * cr,
 	    GtkStateType state_type,
-	    GdkRectangle * area,
 	    GtkWidget * widget,
 	    const gchar * detail, 
             gint y1, 
@@ -76,10 +72,9 @@ glide_draw_vline (GtkStyle * style,
  
 GE_INTERNAL void
 glide_draw_check (GtkStyle * style,
-	    GdkWindow * window,
+	    cairo_t * cr,
 	    GtkStateType state,
 	    GtkShadowType shadow,
-	    GdkRectangle * area,
 	    GtkWidget * widget,
 	    const gchar * detail, 
             gint x, 
@@ -89,10 +84,9 @@ glide_draw_check (GtkStyle * style,
  
 GE_INTERNAL void
 glide_draw_option (GtkStyle * style,
-	     GdkWindow * window,
+	     cairo_t * cr,
 	     GtkStateType state,
 	     GtkShadowType shadow,
-	     GdkRectangle * area,
 	     GtkWidget * widget,
 	     const gchar * detail, 
              gint x,
@@ -102,10 +96,9 @@ glide_draw_option (GtkStyle * style,
  
 GE_INTERNAL void
 glide_draw_arrow (GtkStyle * style,
-	    GdkWindow * window,
+	    cairo_t * cr,
 	    GtkStateType state,
 	    GtkShadowType shadow,
-	    GdkRectangle * area,
 	    GtkWidget * widget,
 	    const gchar * detail,
 	    GtkArrowType arrow_type,
@@ -117,10 +110,9 @@ glide_draw_arrow (GtkStyle * style,
  
 GE_INTERNAL void
 glide_draw_shadow (GtkStyle * style,
-	     GdkWindow * window,
+	     cairo_t * cr,
 	     GtkStateType state_type,
 	     GtkShadowType shadow_type,
-	     GdkRectangle * area,
 	     GtkWidget * widget,
 	     const gchar * detail, 
              gint x, 
@@ -130,10 +122,9 @@ glide_draw_shadow (GtkStyle * style,
  
 GE_INTERNAL void
 glide_draw_box (GtkStyle * style,
-	  GdkWindow * window,
+	  cairo_t * cr,
 	  GtkStateType state_type,
 	  GtkShadowType shadow_type,
-	  GdkRectangle * area,
 	  GtkWidget * widget,
 	  const gchar * detail, 
           gint x, 
@@ -143,10 +134,9 @@ glide_draw_box (GtkStyle * style,
 
 GE_INTERNAL void
 glide_draw_flat_box (GtkStyle * style,
-	  GdkWindow * window,
+	  cairo_t * cr,
 	  GtkStateType state_type,
 	  GtkShadowType shadow_type,
-	  GdkRectangle * area,
 	  GtkWidget * widget,
 	  const gchar * detail, 
           gint x, 
@@ -156,10 +146,9 @@ glide_draw_flat_box (GtkStyle * style,
  
 GE_INTERNAL void
 glide_draw_tab (GtkStyle * style,
-	  GdkWindow * window,
+	  cairo_t * cr,
 	  GtkStateType state,
 	  GtkShadowType shadow,
-	  GdkRectangle * area,
 	  GtkWidget * widget,
 	  const gchar * detail, 
           gint x, 
@@ -169,10 +158,9 @@ glide_draw_tab (GtkStyle * style,
  
 GE_INTERNAL void
 glide_draw_slider (GtkStyle * style,
-	     GdkWindow * window,
+	     cairo_t * cr,
 	     GtkStateType state_type,
 	     GtkShadowType shadow_type,
-	     GdkRectangle * area,
 	     GtkWidget * widget,
 	     const gchar * detail,
 	     gint x,
@@ -183,10 +171,9 @@ glide_draw_slider (GtkStyle * style,
  
 GE_INTERNAL void
 glide_draw_extension (GtkStyle * style,
-		GdkWindow * window,
+		cairo_t * cr,
 		GtkStateType state_type,
 		GtkShadowType shadow_type,
-		GdkRectangle * area,
 		GtkWidget * widget,
 		const gchar * detail,
 		gint x,
@@ -197,10 +184,9 @@ glide_draw_extension (GtkStyle * style,
                 
 GE_INTERNAL void
 glide_draw_handle (GtkStyle * style,
-	     GdkWindow * window,
+	     cairo_t * cr,
 	     GtkStateType state_type,
 	     GtkShadowType shadow_type,
-	     GdkRectangle * area,
 	     GtkWidget * widget,
 	     const gchar * detail,
 	     gint x,
@@ -212,9 +198,8 @@ glide_draw_handle (GtkStyle * style,
 
 GE_INTERNAL void
 glide_draw_focus(GtkStyle *style,
-           GdkWindow *window,
+           cairo_t *cr,
            GtkStateType state_type,
-           GdkRectangle *area,
            GtkWidget *widget,
            const gchar *detail,
            gint x,
@@ -224,10 +209,9 @@ glide_draw_focus(GtkStyle *style,
 
 GE_INTERNAL void
 glide_draw_layout (GtkStyle        *style,
-	     GdkWindow       *window,
+	     cairo_t         *cr,
 	     GtkStateType     state_type,
 	     gboolean         use_text,
-	     GdkRectangle    *area,
 	     GtkWidget       *widget,
 	     const char      *detail,
 	     int              x,
