@@ -118,9 +118,8 @@ mist_draw_border (GtkStyle *style,
 
 static void
 mist_style_draw_hline(GtkStyle *style,
-           GdkWindow *window,
+           cairo_t *cr,
            GtkStateType state_type,
-           GdkRectangle *area,
            GtkWidget *widget,
            const char *detail,
            int x1,
@@ -132,12 +131,9 @@ mist_style_draw_hline(GtkStyle *style,
 	int thickness_light;
 	int thickness_dark;
 	int i;
-	cairo_t *cr;
 
 	CHECK_ARGS
 	
-	cr = ge_gdk_drawable_to_cairo (window, area);
-
 	if (CHECK_DETAIL(detail, "hseparator") 
 	    || CHECK_DETAIL(detail, "menuitem") 
 	    || CHECK_DETAIL(detail, "slider")
@@ -181,15 +177,12 @@ mist_style_draw_hline(GtkStyle *style,
 			cairo_stroke (cr);
 		}
 	}
-	
-	cairo_destroy(cr);
 }
 
 static void
 mist_style_draw_vline(GtkStyle *style,
-           GdkWindow *window,
+           cairo_t *cr,
            GtkStateType state_type,
-           GdkRectangle *area,
            GtkWidget *widget,
            const char *detail,
            int y1,
@@ -201,12 +194,9 @@ mist_style_draw_vline(GtkStyle *style,
 	int thickness_light;
 	int thickness_dark;
 	int i;
-	cairo_t *cr;
 
 	CHECK_ARGS
 	
-	cr = ge_gdk_drawable_to_cairo (window, area);
-	
 	if (CHECK_DETAIL(detail, "vseparator") 
 	    || CHECK_DETAIL(detail, "toolbar") 
 	    || CHECK_DETAIL(detail, "slider") 
@@ -250,16 +240,13 @@ mist_style_draw_vline(GtkStyle *style,
 			cairo_stroke (cr);
 		}
 	}
-	
-	cairo_destroy(cr);
 }
 
 static void
 mist_style_draw_shadow(GtkStyle *style,
-	    GdkWindow *window,
+	    cairo_t *cr,
 	    GtkStateType state_type,
 	    GtkShadowType shadow_type,
-	    GdkRectangle *area,
 	    GtkWidget *widget,
 	    const char *detail,
 	    int x,
@@ -268,18 +255,14 @@ mist_style_draw_shadow(GtkStyle *style,
 	    int height)
 {
 	MistStyle *mist_style = MIST_STYLE (style);
-	cairo_t *cr;
 
 	CHECK_ARGS	
-	SANITIZE_SIZE
 
 	shadow_type = mist_get_shadow_type (style, detail, shadow_type);
 
 	if (shadow_type == GTK_SHADOW_NONE)
 		return;
 
-	cr = ge_gdk_drawable_to_cairo (window, area);
-
 	if (state_type == GTK_STATE_INSENSITIVE) {
 		shadow_type = GTK_SHADOW_ETCHED_IN;
 	}
@@ -294,132 +277,13 @@ mist_style_draw_shadow(GtkStyle *style,
 		mist_draw_border (style, cr, state_type,
 				       shadow_type,x, y, width, height);
 	}
-
-	cairo_destroy(cr);
-}
-
-static void
-mist_style_draw_polygon(GtkStyle *style,
-             GdkWindow *window,
-             GtkStateType state_type,
-             GtkShadowType shadow_type,
-             GdkRectangle *area,
-             GtkWidget *widget,
-             const char *detail,
-             GdkPoint *points,
-             int npoints,
-             int fill)
-{
-	MistStyle *mist_style = MIST_STYLE (style);
-
-	static const gdouble pi_over_4 = G_PI_4;
-	static const gdouble pi_3_over_4 = G_PI_4 * 3;
-	
-	CairoColor           *color1;
-	CairoColor           *color2;
-	CairoColor           *color3;
-	CairoColor           *color4;
-	gdouble            angle;
-	int                xadjust;
-	int                yadjust;
-	int                i;
-	cairo_t           *cr;
-
-	CHECK_ARGS
-	g_return_if_fail(points != NULL);
-	
-	switch (shadow_type) {
-	case GTK_SHADOW_IN:
-		color1 = &mist_style->color_cube.light[state_type];
-		color2 = &mist_style->color_cube.dark[state_type];
-		color3 = &mist_style->color_cube.light[state_type];
-		color4 = &mist_style->color_cube.dark[state_type];
-		break;
-	case GTK_SHADOW_ETCHED_IN:
-		color1 = &mist_style->color_cube.light[state_type];
-		color2 = &mist_style->color_cube.dark[state_type];
-		color3 = &mist_style->color_cube.dark[state_type];
-		color4 = &mist_style->color_cube.light[state_type];
-		break;
-	case GTK_SHADOW_OUT:
-		color1 = &mist_style->color_cube.dark[state_type];
-		color2 = &mist_style->color_cube.light[state_type];
-		color3 = &mist_style->color_cube.dark[state_type];
-		color4 = &mist_style->color_cube.light[state_type];
-		break;
-	case GTK_SHADOW_ETCHED_OUT:
-		color1 = &mist_style->color_cube.dark[state_type];
-		color2 = &mist_style->color_cube.light[state_type];
-		color3 = &mist_style->color_cube.light[state_type];
-		color4 = &mist_style->color_cube.dark[state_type];
-		break;
-	default:
-		return;
-	}
-
-	cr = ge_gdk_drawable_to_cairo (window, area);
-	
-	if (fill)
-		ge_cairo_polygon(cr, &mist_style->color_cube.bg[state_type], points, npoints);
-	
-	npoints--;
-	
-	for (i = 0; i < npoints; i++) {
-		if ((points[i].x == points[i + 1].x) &&
-		    (points[i].y == points[i + 1].y)) {
-			angle = 0;
-		} else {
-			angle = atan2(points[i + 1].y - points[i].y,
-				      points[i + 1].x - points[i].x);
-		}
-		
-		if ((angle > -pi_3_over_4) && (angle < pi_over_4)) {
-			if (angle > -pi_over_4) {
-				xadjust = 0;
-				yadjust = 1;
-			} else {
-				xadjust = 1;
-				yadjust = 0;
-			}
-
-			ge_cairo_line(cr, color1,
-				      points[i].x - xadjust, 
-				      points[i].y - yadjust,
-				      points[i + 1].x - xadjust, 
-				      points[i + 1].y - yadjust);
-			ge_cairo_line(cr, color3,
-				      points[i].x, points[i].y,
-				      points[i + 1].x, points[i + 1].y);
-		}
-		else {
-			if ((angle < -pi_3_over_4) || (angle > pi_3_over_4)) {
-				xadjust = 0;
-				yadjust = 1;
-			} else {
-				xadjust = 1;
-				yadjust = 0;
-			}
-			
-			ge_cairo_line(cr, color4,
-				      points[i].x + xadjust, 
-				      points[i].y + yadjust,
-				      points[i + 1].x + xadjust, 
-				      points[i + 1].y + yadjust);
-			ge_cairo_line(cr, color2,
-				      points[i].x, points[i].y,
-				      points[i + 1].x, points[i + 1].y);
-		}
-	}
-
-	cairo_destroy(cr);
 }
 
 static void
 mist_style_draw_diamond(GtkStyle * style,
-             GdkWindow * window,
+             cairo_t * cr,
              GtkStateType state_type,
              GtkShadowType shadow_type,
-             GdkRectangle * area,
              GtkWidget * widget,
              const char *detail,
              int x,
@@ -431,16 +295,12 @@ mist_style_draw_diamond(GtkStyle * style,
 
 	int half_width;
 	int half_height;
-	cairo_t *cr;
 	
 	CHECK_ARGS
-	SANITIZE_SIZE
 	
 	half_width = width / 2;
 	half_height = height / 2;
 
-	cr = ge_gdk_drawable_to_cairo (window, area);
-
 	switch (shadow_type) {
 	case GTK_SHADOW_IN:
 		ge_cairo_line(cr, &mist_style->color_cube.light[state_type],
@@ -523,16 +383,13 @@ mist_style_draw_diamond(GtkStyle * style,
 	default:
 		break;
 	}
-
-	cairo_destroy(cr);
 }
 
 static void
 mist_style_draw_box(GtkStyle *style,
-         GdkWindow *window,
+         cairo_t *cr,
          GtkStateType state_type,
          GtkShadowType shadow_type,
-         GdkRectangle *area,
          GtkWidget *widget,
          const char *detail,
          int x,
@@ -543,10 +400,8 @@ mist_style_draw_box(GtkStyle *style,
 	MistStyle *mist_style = MIST_STYLE (style);
 
 	CairoColor *light, *dark;
-	cairo_t *cr;
 	
 	CHECK_ARGS
-	SANITIZE_SIZE
 
 	if (CHECK_DETAIL(detail, "menuitem") && state_type == GTK_STATE_PRELIGHT) {
 		state_type = GTK_STATE_SELECTED;
@@ -555,12 +410,10 @@ mist_style_draw_box(GtkStyle *style,
 	light = &mist_style->color_cube.light[state_type];
 	dark = &mist_style->color_cube.dark[state_type];
 
-	cr = ge_gdk_drawable_to_cairo (window, area);
-	
 	if (CHECK_DETAIL(detail, "optionmenutab")) {
 		ge_cairo_line(cr, dark, x - 5, y, x - 5, y + height);
 		
-		gtk_paint_arrow (style, window, state_type, shadow_type, area, 
+		gtk_paint_arrow (style, cr, state_type, shadow_type,
 				 widget, detail, GTK_ARROW_DOWN, TRUE,
 				 x + 1, y + 1, width - 2, height - 2);
 	} else if (CHECK_DETAIL(detail, "trough")) {
@@ -634,9 +487,9 @@ mist_style_draw_box(GtkStyle *style,
 			}
 		}
 		
-		gtk_style_apply_default_background(style, window,
-						   widget && !gtk_widget_get_has_window (widget),
-						   state_type, area,
+		gtk_style_apply_default_background(style, cr,
+						   gtk_widget_get_window (widget),
+						   state_type,
 						   x, y, width, height);
 
 		shadow_type = mist_get_shadow_type (style, detail, shadow_type);
@@ -650,16 +503,13 @@ mist_style_draw_box(GtkStyle *style,
 					       x, y, width, height);
 		}
 	}
-
-	cairo_destroy(cr);
 }
 
 static void
 mist_style_draw_check(GtkStyle *style,
-           GdkWindow *window,
+           cairo_t *cr,
            GtkStateType state_type,
            GtkShadowType shadow_type,
-           GdkRectangle *area,
            GtkWidget *widget,
            const char *detail,
            int x,
@@ -668,11 +518,8 @@ mist_style_draw_check(GtkStyle *style,
            int height)
 {
 	MistStyle *mist_style = MIST_STYLE (style);
-	cairo_t *cr;
 
-	SANITIZE_SIZE
-
-	cr = ge_gdk_drawable_to_cairo (window, area);
+        CHECK_ARGS
 
 	ge_cairo_set_color(cr, &mist_style->color_cube.base[state_type == GTK_STATE_INSENSITIVE ? GTK_STATE_INSENSITIVE : GTK_STATE_NORMAL ]);	
 	cairo_rectangle(cr, x + 1, y + 1, width - 3, height - 3);
@@ -691,41 +538,43 @@ mist_style_draw_check(GtkStyle *style,
 	} else if (shadow_type == GTK_SHADOW_ETCHED_IN) { /* inconsistent */
 #define gray50_width 2
 #define gray50_height 2
-		GdkBitmap *stipple;
-		GdkGC *gc = style->base_gc[GTK_STATE_SELECTED];
-		static const char gray50_bits[] = {
-			0x02, 0x01
+		cairo_surface_t *surface;
+		cairo_pattern_t *pattern;
+		static guchar gray50_bits[] = {
+			/* pixman expects strides to be multiples of 4 */
+			0x02, 0x01, 0x00, 0x00,
+			0x02, 0x01, 0x00, 0x00
 		};
 
-		stipple = gdk_bitmap_create_from_data (window,
-						       gray50_bits, 
-						       gray50_width,
-						       gray50_height);
-  
-		if (area)
-			gdk_gc_set_clip_rectangle (gc, area);
+		surface = cairo_image_surface_create_for_data (gray50_bits,
+		                                               CAIRO_FORMAT_A8,
+		                                               gray50_width,
+		                                               gray50_height,
+		                                               4);
+		pattern = cairo_pattern_create_for_surface (surface);
 
-		gdk_gc_set_fill (gc, GDK_STIPPLED);
-		gdk_gc_set_stipple (gc, stipple);
-		gdk_draw_rectangle(window, gc,
-				   TRUE, x + 2, y + 2, width - 5, height - 5);
-		gdk_gc_set_fill (gc, GDK_SOLID);
+		cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
+		cairo_pattern_set_filter (pattern, CAIRO_FILTER_NEAREST);
 
-		if (area)
-			gdk_gc_set_clip_rectangle (gc, NULL);
+		gdk_cairo_set_source_color (cr,
+		                            &style->base[GTK_STATE_SELECTED]);
+
+		cairo_mask (cr, pattern);
+		cairo_rectangle (cr, x + 2, y + 2, width - 5, height - 5);
+		cairo_fill (cr);
+
+		cairo_surface_destroy (surface);
+		cairo_pattern_destroy (pattern);
 #undef gray50_width
 #undef gray50_height 
 	}
-
-	cairo_destroy(cr);
 }
 
 static void
 mist_style_draw_option(GtkStyle *style,
-            GdkWindow *window,
+            cairo_t *cr,
             GtkStateType state_type,
             GtkShadowType shadow_type,
-            GdkRectangle *area,
             GtkWidget *widget,
             const char *detail,
             int x,
@@ -734,11 +583,8 @@ mist_style_draw_option(GtkStyle *style,
             int height)
 {
 	MistStyle *mist_style = MIST_STYLE (style);
-	cairo_t *cr;
 
-	SANITIZE_SIZE
-
-	cr = ge_gdk_drawable_to_cairo (window, area);
+        CHECK_ARGS
 
 	cairo_arc(cr, x + floor(width/2), y + floor(height/2), floor(width/2) - 0.5, 0 , 2 * G_PI);
 	ge_cairo_set_color(cr, &mist_style->color_cube.dark[state_type == GTK_STATE_INSENSITIVE ? GTK_STATE_INSENSITIVE : GTK_STATE_NORMAL]);	
@@ -780,16 +626,13 @@ mist_style_draw_option(GtkStyle *style,
 		cairo_arc(cr, x + floor(width/2), y + floor(height/2), floor((width - 7)/2) + 1, 0 , 2 * G_PI);
 		cairo_fill (cr);
 	}
-
-	cairo_destroy(cr);
 }
 
 static void
 mist_style_draw_shadow_gap(GtkStyle *style,
-                GdkWindow *window,
+                cairo_t *cr,
                 GtkStateType state_type,
                 GtkShadowType shadow_type,
-                GdkRectangle *area,
                 GtkWidget *widget,
                 const char *detail,
                 int x,
@@ -804,20 +647,14 @@ mist_style_draw_shadow_gap(GtkStyle *style,
 
 	CairoColor *color1 = NULL;
 	CairoColor *color2 = NULL;
-	cairo_t *cr;
 	gint start;
 	
-	g_return_if_fail (window != NULL);
+	CHECK_ARGS
 	
-	SANITIZE_SIZE
 	shadow_type = mist_get_shadow_type (style, detail, shadow_type);
 	
-	cr = ge_gdk_drawable_to_cairo (window, area);
-
 	switch (shadow_type) {
 	case GTK_SHADOW_NONE:
-		/* need to destroy the cairo context. */
-		cairo_destroy (cr);
 		return;
 	case GTK_SHADOW_IN:
 		color1 = &mist_style->color_cube.dark[state_type];
@@ -857,16 +694,13 @@ mist_style_draw_shadow_gap(GtkStyle *style,
 	cairo_clip (cr);
 	cairo_new_path (cr);
 	ge_cairo_simple_border (cr, color1, color2, x, y, width, height, FALSE);
-	
-	cairo_destroy(cr);
 }
 
 static void
 mist_style_draw_box_gap(GtkStyle *style,
-	     GdkWindow *window,
+	     cairo_t *cr,
 	     GtkStateType state_type,
 	     GtkShadowType shadow_type,
-	     GdkRectangle *area,
 	     GtkWidget *widget,
 	     const char *detail,
 	     int x,
@@ -877,24 +711,23 @@ mist_style_draw_box_gap(GtkStyle *style,
 	     int gap_x,
 	     int gap_width)
 {
-	SANITIZE_SIZE
+	CHECK_ARGS
 
-	gtk_style_apply_default_background(style, window,
-					   widget && !gtk_widget_get_has_window (widget),
-					   state_type, area,
+	gtk_style_apply_default_background(style, cr,
+					   gtk_widget_get_window (widget),
+					   state_type,
 					   x, y, width, height);
 
-	mist_style_draw_shadow_gap (style, window, state_type, shadow_type,
-			 area, widget, detail, x, y, width, height, 
+	mist_style_draw_shadow_gap (style, cr, state_type, shadow_type,
+			 widget, detail, x, y, width, height, 
 			 gap_side, gap_x, gap_width);
 }
 
 static void
 mist_style_draw_extension(GtkStyle *style,
-               GdkWindow *window,
+               cairo_t *cr,
                GtkStateType state_type,
                GtkShadowType shadow_type,
-               GdkRectangle *area,
                GtkWidget *widget,
                const char *detail,
                int x,
@@ -903,34 +736,21 @@ mist_style_draw_extension(GtkStyle *style,
                int height,
                GtkPositionType gap_side)
 {
-	cairo_t *canvas;
 	CairoColor *background, *light, *dark;
 
 	CHECK_ARGS
-	SANITIZE_SIZE
-
 
 	background = &MIST_STYLE(style)->color_cube.bg[state_type];
 	light = &MIST_STYLE(style)->color_cube.light[state_type];
 	dark = &MIST_STYLE(style)->color_cube.dark[state_type];
 	
-	canvas = ge_gdk_drawable_to_cairo (window, area);
-
-	if ((!style->bg_pixmap[state_type]) || GDK_IS_PIXMAP(window)) 
-	{
-		ge_cairo_set_color(canvas, background);	
-
-		cairo_rectangle(canvas, x, y, width, height);
-		cairo_fill(canvas);
-	} else {
-		gtk_style_apply_default_background
-			(style, window,
-			 widget && !gtk_widget_get_has_window (widget),
-			 state_type, area, x, y, width, height);
-	}
+        gtk_style_apply_default_background
+                (style, cr,
+                 gtk_widget_get_window (widget),
+                 state_type, x, y, width, height);
 	
-	cairo_rectangle (canvas, x, y, width, height);
-	cairo_clip (canvas);
+	cairo_rectangle (cr, x, y, width, height);
+	cairo_clip (cr);
 	
 	switch(gap_side) {
 	case GTK_POS_BOTTOM:
@@ -949,16 +769,14 @@ mist_style_draw_extension(GtkStyle *style,
 		break;
 	}
 
-	ge_cairo_simple_border (canvas, light, dark, x, y, width, height, FALSE);
-	cairo_destroy(canvas);
+	ge_cairo_simple_border (cr, light, dark, x, y, width, height, FALSE);
 }
 
 static void
 mist_style_draw_handle(GtkStyle *style,
-            GdkWindow *window,
+            cairo_t *cr,
             GtkStateType state_type,
             GtkShadowType shadow_type,
-            GdkRectangle *area,
             GtkWidget *widget,
             const char *detail,
             int x,
@@ -972,13 +790,11 @@ mist_style_draw_handle(GtkStyle *style,
 	CairoColor *light, *dark;
 	GdkRectangle dest;
 	int modx, mody;
-	cairo_t *cr;
 	
 	CHECK_ARGS
-	SANITIZE_SIZE
 
 	if (!ge_is_panel_widget_item (widget) && !ge_object_is_a ((GObject*)widget, "PanelToplevel"))
-		gtk_paint_box(style, window, state_type, shadow_type, area, widget,
+		gtk_paint_box(style, cr, state_type, shadow_type, widget,
 			      detail, x, y, width, height);
 	
 	light = &mist_style->color_cube.light[state_type];
@@ -1001,8 +817,6 @@ mist_style_draw_handle(GtkStyle *style,
 		modx = 0; mody = 4;
 	}
 	
-	cr = ge_gdk_drawable_to_cairo (window, &dest);
-
 	mist_dot(cr,
 		 light, dark,
 		 x + width / 2 - modx,
@@ -1015,15 +829,12 @@ mist_style_draw_handle(GtkStyle *style,
 		 light, dark,
 		 x + width / 2 + modx,
 		 y + height / 2 + mody);
-	
-	cairo_destroy(cr);
 }
 
 static void
 mist_style_draw_resize_grip(GtkStyle *style,
-		 GdkWindow *window,
+		 cairo_t *cr,
 		 GtkStateType state_type,
-		 GdkRectangle *area,
 		 GtkWidget *widget,
 		 const char *detail,
 		 GdkWindowEdge edge,
@@ -1038,10 +849,8 @@ mist_style_draw_resize_grip(GtkStyle *style,
 	int xi, yi;
 	int max_x, max_y;
 	int threshold;
-	cairo_t *cr;
 	
 	CHECK_ARGS
-	SANITIZE_SIZE
 	
 	switch (edge) {
 	case GDK_WINDOW_EDGE_NORTH_WEST:
@@ -1109,8 +918,9 @@ mist_style_draw_resize_grip(GtkStyle *style,
 		g_assert_not_reached ();
 	}
 	
-	gtk_style_apply_default_background (style, window, FALSE,
-					    state_type, area,
+	gtk_style_apply_default_background (style, cr,
+                                            gtk_widget_get_window (widget),
+					    state_type,
 					    x, y, width, height);
 
 	light = &mist_style->color_cube.light[state_type];
@@ -1121,8 +931,6 @@ mist_style_draw_resize_grip(GtkStyle *style,
 	
 	threshold = max_x;
 	
-	cr = ge_gdk_drawable_to_cairo (window, area);
-
 	for (xi = 0; xi <= max_x; xi++) {
 		for (yi = 0; yi <= max_y; yi++) {
 			gboolean draw_dot;
@@ -1158,37 +966,28 @@ mist_style_draw_resize_grip(GtkStyle *style,
 			}
 		}
 	}
-
-	cairo_destroy(cr);
 }
 
 static void
 mist_style_draw_layout (GtkStyle        *style,
-	     GdkWindow       *window,
+	     cairo_t         *cr,
 	     GtkStateType     state_type,
 	     gboolean         use_text,
-	     GdkRectangle    *area,
 	     GtkWidget       *widget,
 	     const char      *detail,
 	     int              x,
 	     int              y,
 	     PangoLayout      *layout)
 {
-	GdkGC *gc;
-	
 	CHECK_ARGS
+
+	if (use_text)
+		gdk_cairo_set_source_color (cr, &style->text[state_type]);
+	else
+		gdk_cairo_set_source_color (cr, &style->fg[state_type]);
 	
-	gc = use_text ? style->text_gc[state_type] : style->fg_gc[state_type];
-	
-	if (area) {
-		gdk_gc_set_clip_rectangle (gc, area);
-	}
-	
-	gdk_draw_layout (window, gc, x, y, layout);
-	
-	if (area) {
-		gdk_gc_set_clip_rectangle (gc, NULL);
-	}
+	ge_cairo_transform_for_layout (cr, layout, x, y);
+	pango_cairo_show_layout (cr, layout);
 }
 
 static GdkPixbuf *
@@ -1269,8 +1068,8 @@ mist_style_render_icon (GtkStyle            *style,
 	if (widget && gtk_widget_has_screen (widget)) {
 		screen = gtk_widget_get_screen (widget);
 		settings = gtk_settings_get_for_screen (screen);
-	} else if (style->colormap) {
-		screen = gdk_colormap_get_screen (style->colormap);
+	} else if (style->visual) {
+		screen = gdk_visual_get_screen (style->visual);
 		settings = gtk_settings_get_for_screen (screen);
 	} else {
 		settings = gtk_settings_get_default ();
@@ -1319,9 +1118,8 @@ mist_style_render_icon (GtkStyle            *style,
 
 static void
 mist_style_draw_focus (GtkStyle *style,
-		       GdkWindow *window,
+		       cairo_t *cr,
 		       GtkStateType state_type,
-		       GdkRectangle *area,
 		       GtkWidget *widget,
 		       const gchar *detail,
 		       gint x,
@@ -1331,12 +1129,9 @@ mist_style_draw_focus (GtkStyle *style,
 {
 	MistStyle *mist_style = MIST_STYLE (style);
 	CairoColor *dark;
-	cairo_t *cr;
 
 	CHECK_ARGS
-	SANITIZE_SIZE
 
-	cr = ge_gdk_drawable_to_cairo (window, area);
 	cairo_translate (cr, 0.5, 0.5);
 	width--; height--;
 
@@ -1344,7 +1139,6 @@ mist_style_draw_focus (GtkStyle *style,
 	ge_cairo_set_color (cr, dark);
 	cairo_rectangle (cr, x, y, width, height);
 	cairo_stroke (cr);
-	cairo_destroy (cr);
 }
 
 static void
@@ -1372,7 +1166,6 @@ mist_style_class_init (MistStyleClass *klass)
 	style_class->draw_hline = mist_style_draw_hline;
 	style_class->draw_vline = mist_style_draw_vline;
 	style_class->draw_shadow = mist_style_draw_shadow;
-	style_class->draw_polygon = mist_style_draw_polygon;
 	style_class->draw_diamond = mist_style_draw_diamond;
 	style_class->draw_box = mist_style_draw_box;
 	style_class->draw_tab = mist_style_draw_box;
