@@ -1,3 +1,34 @@
+/* Helper functions for gtk-engines
+ *
+ * Copyright (C) 2006 Andrew Johnson <acjgenius@earthlink.net>
+ * Copyright (C) 2006 Thomas Wood <thos@gnome.org>
+ * Copyright (C) 2006-2007 Benjamin Berg <benjamin@sipsolutions.net>
+ * Copyright (C) 2007 Andrea Cimitan <andrea.cimitan@gmail.com>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ * Project contact: <gnome-themes-list@gnome.org>
+ *
+ *
+ * Written by Andrew Johnson <acjgenius@earthlink.net>
+ * Written by Benjamin Berg <benjamin@sipsolutions.net>
+ * Written by Thomas Wood <thos@gnome.org>
+ * modified by Andrea Cimitan <andrea.cimitan@gmail.com>
+ *
+ */
+
 #include <math.h>
 #include "general-support.h"
 #include "cairo-support.h"
@@ -244,28 +275,6 @@ ge_mix_color (const CairoColor *color1, const CairoColor *color2,
 	composite->a = 1.0;
 }
 
-cairo_t * 
-ge_gdk_drawable_to_cairo (GdkDrawable  *window, GdkRectangle *area)
-{
-	cairo_t *cr;
-
-	g_return_val_if_fail (window != NULL, NULL);
-
-	cr = (cairo_t*) gdk_cairo_create (window);
-	cairo_set_line_width (cr, 1.0);
-	cairo_set_line_cap (cr, CAIRO_LINE_CAP_SQUARE);
-	cairo_set_line_join (cr, CAIRO_LINE_JOIN_MITER);
-
-	if (area)
-	{
-		cairo_rectangle (cr, area->x, area->y, area->width, area->height);
-		cairo_clip_preserve (cr);
-		cairo_new_path (cr);
-	}
-
-	return cr;
-}
-
 void 
 ge_cairo_set_color (cairo_t *cr, const CairoColor *color)
 {
@@ -704,10 +713,6 @@ ge_cairo_color_pattern(CairoColor *base)
 {
 	CairoPattern * result = g_new0(CairoPattern, 1);
 
-	#if  ((CAIRO_VERSION_MAJOR < 1) || ((CAIRO_VERSION_MAJOR == 1) && (CAIRO_VERSION_MINOR < 2)))
-		result->type = CAIRO_PATTERN_TYPE_SOLID;
-	#endif
-
 	result->scale = GE_DIRECTION_NONE;
 	result->translate = GE_DIRECTION_NONE;
 
@@ -729,19 +734,13 @@ ge_cairo_color_pattern(CairoColor *base)
 CairoPattern*
 ge_cairo_pixbuf_pattern(GdkPixbuf *pixbuf)
 {
-	CairoPattern * result = g_new0(CairoPattern, 1);
+	CairoPattern * result;
+        cairo_pattern_t *pattern;
 
 	cairo_t *canvas;
 	cairo_surface_t * surface;
 	gint width, height;
 
-	#if ((CAIRO_VERSION_MAJOR < 1) || ((CAIRO_VERSION_MAJOR == 1) && (CAIRO_VERSION_MINOR < 2)))
-		result->type = CAIRO_PATTERN_TYPE_SURFACE;
-	#endif
-
-	result->scale = GE_DIRECTION_NONE;
-	result->translate = GE_DIRECTION_BOTH;
-
 	width = gdk_pixbuf_get_width(pixbuf);
 	height = gdk_pixbuf_get_height(pixbuf);
 	
@@ -754,38 +753,26 @@ ge_cairo_pixbuf_pattern(GdkPixbuf *pixbuf)
 	cairo_fill (canvas);
 	cairo_destroy(canvas);
 
-	result->handle = cairo_pattern_create_for_surface (surface);
+	pattern = cairo_pattern_create_for_surface (surface);
 	cairo_surface_destroy(surface);
 
-	cairo_pattern_set_extend (result->handle, CAIRO_EXTEND_REPEAT);
+	cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
 
-	result->operator = CAIRO_OPERATOR_SOURCE;
+        result = ge_cairo_pattern_pattern (pattern);
+        cairo_pattern_destroy (pattern);
 
 	return result;
 }
 
-/***********************************************
- * ge_cairo_pixmap_pattern -
- *  
- *   Create A Tiled Pixmap Pattern
- ***********************************************/
 CairoPattern*
-ge_cairo_pixmap_pattern(GdkPixmap *pixmap)
-{	
-	CairoPattern * result = NULL;
+ge_cairo_pattern_pattern(cairo_pattern_t *pattern)
+{
+	CairoPattern * result = g_new0(CairoPattern, 1);
 
-	GdkPixbuf * pixbuf;
-	gint width, height;
-
-	gdk_drawable_get_size (GDK_DRAWABLE (pixmap), &width, &height);
-
-	pixbuf = gdk_pixbuf_get_from_drawable(NULL, GDK_DRAWABLE (pixmap), 
-	         gdk_drawable_get_colormap(GDK_DRAWABLE (pixmap)), 
-	         0, 0, 0, 0, width, height);
-
-	result = ge_cairo_pixbuf_pattern(pixbuf);
-	
-	g_object_unref (pixbuf);
+	result->scale = GE_DIRECTION_NONE;
+	result->translate = GE_DIRECTION_BOTH;
+        result->handle = cairo_pattern_reference (pattern);
+	result->operator = CAIRO_OPERATOR_SOURCE;
 
 	return result;
 }
@@ -805,10 +792,6 @@ ge_cairo_linear_shade_gradient_pattern(CairoColor *base,
                                        gboolean vertical)
 {
 	CairoPattern * result = g_new0(CairoPattern, 1);
-	
-	#if  ((CAIRO_VERSION_MAJOR < 1) || ((CAIRO_VERSION_MAJOR == 1) && (CAIRO_VERSION_MINOR < 2)))
-		result->type = CAIRO_PATTERN_TYPE_LINEAR;
-	#endif
 
 	if (vertical)
 	{
@@ -844,6 +827,38 @@ ge_cairo_pattern_destroy(CairoPattern *pattern)
 	}
 }
 
+void
+ge_cairo_transform_for_layout (cairo_t *cr,
+                               PangoLayout *layout,
+                               int x,
+                               int y)
+{
+        const PangoMatrix *matrix;
+
+        matrix = pango_context_get_matrix (pango_layout_get_context (layout));
+        if (matrix)
+        {
+                cairo_matrix_t cairo_matrix;
+                PangoRectangle rect;
+                
+                cairo_matrix_init (&cairo_matrix,
+                                   matrix->xx, matrix->yx,
+                                   matrix->xy, matrix->yy,
+                                   matrix->x0, matrix->y0);
+
+                pango_layout_get_extents (layout, NULL, &rect);
+                pango_matrix_transform_rectangle (matrix, &rect);
+                pango_extents_to_pixels (&rect, NULL);
+                                                    
+                cairo_matrix.x0 += x - rect.x;
+                cairo_matrix.y0 += y - rect.y;
+
+                cairo_set_matrix (cr, &cairo_matrix);
+        }
+        else
+                cairo_translate (cr, x, y);
+}
+
 /* 
  * The following function will be called by GTK+ when the module
  * is loaded and checks to see if we are compatible with the
