@@ -32,6 +32,7 @@ typedef enum {
 	WIDGET_TREE_VIEW,
 	WIDGET_TREE_VIEW_HEADER,
 	WIDGET_SCALE,
+	WIDGET_ENTRY,
 	WIDGET_PROGRESS_BAR,
 	WIDGET_MENUBAR,
 	WIDGET_MENUBAR_ITEM,
@@ -56,6 +57,7 @@ widget_type_get_type (void)
 			{ WIDGET_TREE_VIEW, "WIDGET_TREE_VIEW", "GtkTreeView" },
 			{ WIDGET_TREE_VIEW_HEADER, "WIDGET_TREE_VIEW_HEADER", "GtkTreeView-header" },
 			{ WIDGET_SCALE, "WIDGET_SCALE", "GtkScale" },
+			{ WIDGET_ENTRY, "WIDGET_ENTRY", "GtkEntry" },
 			{ WIDGET_PROGRESS_BAR, "WIDGET_PROGRESS_BAR", "GtkProgessBar" },
 			{ WIDGET_MENUBAR, "WIDGET_MENUBAR", "GtkMenubar" },
 			{ WIDGET_MENUBAR_ITEM, "WIDGET_MENUBAR_ITEM", "GtkMenubar-item" },
@@ -338,6 +340,19 @@ static Test tests[] = {
 		GTK_EXPANDER_EXPANDED,
 		GDK_WINDOW_EDGE_SOUTH
 	},
+	{
+		FUNCTION_BOX,
+		WIDGET_ENTRY,
+		"NULL:entry:entry_bg:entry-progress",
+		STATE_ALL,
+		SHADOW_ALL,
+		GTK_ARROW_UP,
+		TRUE,
+		GTK_POS_TOP,
+		GTK_ORIENTATION_HORIZONTAL,
+		GTK_EXPANDER_EXPANDED,
+		GDK_WINDOW_EDGE_SOUTH
+	},
 	{
 		FUNCTION_BOX | FUNCTION_SLIDER,
 		WIDGET_SCROLLBAR,
@@ -511,66 +526,69 @@ end:
 static void
 run_functions (Test *test)
 {
-	GtkStyle *style = window->style;
-	GdkWindow *mywindow = window->window;
+	GtkStyle *style = gtk_widget_get_style (window);
+	GdkWindow *mywindow = gtk_widget_get_window (window);
+	cairo_t *cr = gdk_cairo_create (mywindow);
 	
 	if (widgets[test->widget]) {
-		style = widgets[test->widget]->style;
-		mywindow = widgets[test->widget]->window;
+		style = gtk_widget_get_style (widgets[test->widget]);
+		mywindow = gtk_widget_get_window (widgets[test->widget]);
 	}
 	
 	if (test->function & FUNCTION_ARROW)
-		gtk_paint_arrow (style, mywindow, test->state, test->shadow, NULL, widgets[test->widget], test->detail, test->arrow_type, test->fill, 0, 0, 10, 10);
+		gtk_paint_arrow (style, cr, test->state, test->shadow, widgets[test->widget], test->detail, test->arrow_type, test->fill, 0, 0, 10, 10);
 
 	if (test->function & FUNCTION_BOX)
-		gtk_paint_box (style, mywindow, test->state, test->shadow, NULL, widgets[test->widget], test->detail, 0, 0, 10, 10);
+		gtk_paint_box (style, cr, test->state, test->shadow, widgets[test->widget], test->detail, 0, 0, 10, 10);
 
 	if (test->function & FUNCTION_SHADOW)
-		gtk_paint_shadow (style, mywindow, test->state, test->shadow, NULL, widgets[test->widget], test->detail, 0, 0, 10, 10);
+		gtk_paint_shadow (style, cr, test->state, test->shadow, widgets[test->widget], test->detail, 0, 0, 10, 10);
 
 	if (test->function & FUNCTION_BOX_GAP)
-		gtk_paint_box_gap (style, mywindow, test->state, test->shadow, NULL, widgets[test->widget], test->detail, 0, 0, 10, 10, test->gap_side, 0, 100);
+		gtk_paint_box_gap (style, cr, test->state, test->shadow, widgets[test->widget], test->detail, 0, 0, 10, 10, test->gap_side, 0, 100);
 
 	if (test->function & FUNCTION_SHADOW_GAP)
-		gtk_paint_shadow_gap (style, mywindow, test->state, test->shadow, NULL, widgets[test->widget], test->detail, 0, 0, 10, 10, test->gap_side, 0, 100);
+		gtk_paint_shadow_gap (style, cr, test->state, test->shadow, widgets[test->widget], test->detail, 0, 0, 10, 10, test->gap_side, 0, 100);
 
 
 	if (test->function & FUNCTION_CHECK)
-		gtk_paint_check (style, mywindow, test->state, test->shadow, NULL, widgets[test->widget], test->detail, 0, 0, 10, 10);
+		gtk_paint_check (style, cr, test->state, test->shadow, widgets[test->widget], test->detail, 0, 0, 10, 10);
 		
 	if (test->function & FUNCTION_EXPANDER)
-		gtk_paint_expander (style, mywindow, test->state, NULL, widgets[test->widget], test->detail, 10, 10, test->expander_style);
+		gtk_paint_expander (style, cr, test->state, widgets[test->widget], test->detail, 10, 10, test->expander_style);
 
 	if (test->function & FUNCTION_EXTENSION)
-		gtk_paint_extension (style, mywindow, test->state, test->shadow, NULL, widgets[test->widget], test->detail, 0, 0, 10, 10, test->gap_side);
+		gtk_paint_extension (style, cr, test->state, test->shadow, widgets[test->widget], test->detail, 0, 0, 10, 10, test->gap_side);
 
 	if (test->function & FUNCTION_FLAT_BOX)
-		gtk_paint_flat_box (style, mywindow, test->state, test->shadow, NULL, widgets[test->widget], test->detail, 0, 0, 10, 10);
+		gtk_paint_flat_box (style, cr, test->state, test->shadow, widgets[test->widget], test->detail, 0, 0, 10, 10);
 
 	if (test->function & FUNCTION_FOCUS)
-		gtk_paint_focus (style, mywindow, test->state, NULL, widgets[test->widget], test->detail, 0, 0, 10, 10);
+		gtk_paint_focus (style, cr, test->state, widgets[test->widget], test->detail, 0, 0, 10, 10);
 
 	if (test->function & FUNCTION_HANDLE)
-		gtk_paint_handle (style, mywindow, test->state, test->shadow, NULL, widgets[test->widget], test->detail, 0, 0, 10, 10, test->orientation);
+		gtk_paint_handle (style, cr, test->state, test->shadow, widgets[test->widget], test->detail, 0, 0, 10, 10, test->orientation);
 
 	if (test->function & FUNCTION_OPTION)
-		gtk_paint_option (style, mywindow, test->state, test->shadow, NULL, widgets[test->widget], test->detail, 0, 0, 10, 10);
+		gtk_paint_option (style, cr, test->state, test->shadow, widgets[test->widget], test->detail, 0, 0, 10, 10);
 
 	if (test->function & FUNCTION_RESIZE_GRIP)
-		gtk_paint_resize_grip (style, mywindow, test->state, NULL, widgets[test->widget], test->detail, test->edge, 0, 0, 10, 10);
+		gtk_paint_resize_grip (style, cr, test->state, widgets[test->widget], test->detail, test->edge, 0, 0, 10, 10);
 
 	if (test->function & FUNCTION_SLIDER)
-		gtk_paint_slider (style, mywindow, test->state, test->shadow, NULL, widgets[test->widget], test->detail, 0, 0, 10, 10, test->orientation);
+		gtk_paint_slider (style, cr, test->state, test->shadow, widgets[test->widget], test->detail, 0, 0, 10, 10, test->orientation);
 
 	if (test->function & FUNCTION_TAB)
-		gtk_paint_tab (style, mywindow, test->state, test->shadow, NULL, widgets[test->widget], test->detail, 0, 0, 10, 10);
+		gtk_paint_tab (style, cr, test->state, test->shadow, widgets[test->widget], test->detail, 0, 0, 10, 10);
 
 
 	if (test->function & FUNCTION_HLINE)
-		gtk_paint_hline (style, mywindow, test->state, NULL, widgets[test->widget], test->detail, 1, 10, 4);
+		gtk_paint_hline (style, cr, test->state, widgets[test->widget], test->detail, 1, 10, 4);
 
 	if (test->function & FUNCTION_VLINE)
-		gtk_paint_vline (style, mywindow, test->state, NULL, widgets[test->widget], test->detail, 1, 10, 4);
+		gtk_paint_vline (style, cr, test->state, widgets[test->widget], test->detail, 1, 10, 4);
+
+	cairo_destroy (cr);
 }
 
 static void
@@ -614,11 +632,11 @@ create_testwidgets ()
 	gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), GTK_TREE_MODEL (store));
 
 	widgets[WIDGET_TREE_VIEW] = treeview;
-	/* lets abuse the private API ... */
-	widgets[WIDGET_TREE_VIEW_HEADER] = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), 0)->button;
+	widgets[WIDGET_TREE_VIEW_HEADER] = gtk_widget_get_parent (gtk_widget_get_parent (gtk_tree_view_column_get_widget (gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), 0))));
 	
 	
 	widgets[WIDGET_SCALE] = gtk_hscale_new_with_range (0, 1, 0.1);
+	widgets[WIDGET_ENTRY] = gtk_entry_new ();
 	widgets[WIDGET_PROGRESS_BAR] = gtk_progress_bar_new ();
 	widgets[WIDGET_MENUBAR] = gtk_menu_bar_new ();
 	widgets[WIDGET_MENUBAR_ITEM] = gtk_menu_item_new_with_label ("blah");
@@ -626,8 +644,9 @@ create_testwidgets ()
 	widgets[WIDGET_TOOLBAR] = gtk_toolbar_new ();
 	widgets[WIDGET_DEFAULT_BUTTON] = gtk_button_new_with_label ("blah");
 	/* nasty but *shrug* */
-	GTK_WIDGET_SET_FLAGS(widgets[WIDGET_DEFAULT_BUTTON], GTK_CAN_DEFAULT);
-	GTK_WIDGET_SET_FLAGS(widgets[WIDGET_DEFAULT_BUTTON], GTK_HAS_DEFAULT);
+	gtk_widget_set_can_default (widgets[WIDGET_DEFAULT_BUTTON], TRUE);
+	/* XXX: How to do this without the API? Is it even possible? */
+	/*gtk_widget_set_has_default (widgets[WIDGET_DEFAULT_BUTTON], TRUE);*/
 }
 
 int
@@ -650,7 +669,7 @@ main (int argc, char **argv)
 	gtk_container_add ((GtkContainer*)window, box);
 	
 	for (i = 0; i < WIDGET_COUNT; i++) {
-		if (widgets[i] && !widgets[i]->parent)
+		if (widgets[i] && !gtk_widget_get_parent (widgets[i]))
 			gtk_box_pack_end (GTK_BOX (box), widgets[i], FALSE, FALSE, 0);
 	}
 
@@ -668,12 +687,25 @@ main (int argc, char **argv)
 	 * and destruction of all engine objects. */
         gtk_rc_set_default_files (new_default_files);
 	gtk_rc_reparse_all_for_settings (settings, TRUE);
-	
+
+	gtk_widget_hide (window);
+	gtk_widget_unmap (window);
+
+	while (gdk_events_pending ())
+		gtk_main_iteration_do (FALSE);
+
+	gtk_widget_destroy (window);
+
 	while (gdk_events_pending ())
 		gtk_main_iteration_do (FALSE);
 
 	/* TODO: It would be really cool to check if there is no
 	 * loaded engine at this point. */
 
+	/*
+	 * HRM: Calling the follwing causes an assertion failure inside cairo here -- 01.10.2010, Benjamin
+	cairo_debug_reset_static_data ();
+	*/
+
 	return 0;
 }
